c  F_ADD.FOR
C
c  To add or subtract one coefficient file from another.
c
c  In complex frequency domain, modifies fft coefficient file "fftfil.cof"
c  (obtained  from FFTFIL using the icoef option) according to user-supplied
c  code, folds coefficients into lower quadrants  to preserve appropriate
c  symmetries for real-valued functions in time (space) domain, and outputs
c  new "fftfil.ncf" file, which can be renamed "fftfil.cof" and then
c  inverse-transformed by means of FFTFIL and use of the icoef=-1 option.
c  This program follows the blocking of 'NEWFFTFIL'.
c
c  All user-supplied changes occur in subroutine MODIFY, at several locations
c  indicated by "SUPPLY".  The main program just accommodates FFTFIL's
c  restructuring of the grid. 
c
c  See comments in subroutine MODIFY regarding switch of coordinate
c  axes from x=east, y=north; to x=north, y=east.
c
c  Lin Cordell, 9 Dec '87
c  Revised for PC 21 Aug '90.
c  Fft repair made 1 Feb 91.
c
      character title*56,pgm*8
      character*50 ingrd,ifile1,ifile2,ofile
      dimension g1(2,2048),g2(2,2048)
      data lnri/16/
      ierror=0
      write(6,10)
10    format(1x,'Enter grid from which Fourier coeff.s created:'/1x'*'$)
      read(5,100)ingrd
100   format(a50)
      open(10,file=ingrd,status='old',form='unformatted')
      read(10)title,pgm,n2,n1,nz,yo,dy,xo,dx
c  Note: n2,n1 and dx,dy rotate sense of rows and cols. Now x=north,
c   y=east.
      close(10)
      write(6,7)
7     format(' Enter no. of rows & cols added to grid(nadd):'/1x'*'$)
      read(5,*)nadd
      write(6,50)
50    format(' Enter input coefficient file1 name:'/' *'$)
      read(5,100)ifile1
      write(6,51)
51    format(' Enter input coefficient file2 name:'/' *'$)
      read(5,100)ifile2
      write(6,52)
52    format(' Enter output coefficient file name:'/' *'$)
      read(5,100)ofile
54    write(6,53)
53    format(' Enter operator (1 for plus, -1 for minus:'/' *'$)
      read*,iopr
      if(iopr.ne.1.and.iopr.ne.-1)then
        print*,' Cant handle it.'
        go to 54
      endif
      icmd=10
      l=lnri+1
      lnri21=lnri/2+1
c  set no. of rows for fft: need m=l*2**k, m.gt. or .eq. nx+2*nadd.
c  m=no. of rows, l=no. from 9-16, k=interger.
      n1=n1+2*nadd
  190 l=l-1
      if(l.lt.lnri21) go to 260
      mr=n1/l+0.0000001
      k=1
      idiv=2
  200 iquot=mr/idiv+0.0000001
      if(iquot.lt.idiv) go to 210
      k=k+1
      mr=iquot
      go to 200
  210 k=k+1
      m=l*2**k
  220 mtest=l*2**(k-1)
      if(mtest.lt.n1) go to 230
      k=k-1
      m=mtest
      if(k.eq.0) go to 230
      go to 220
  230 lnxa=m-n1
      if(l.ne.lnri) go to 250
      nxa16=lnxa
  240 nri=l
      nxa=lnxa
      go to 190
  250 if(lnxa.ge.nxa) go to 190
      go to 240
  260 n1=n1+nxa
c  check to see if row block size of 16 will be more efficient
      n116=n1-nxa+nxa16
      ntest=0.9*n116
      if(ntest.gt.n1.or.n116.gt.2048) go to 270
      n1=n1-nxa+nxa16
      nxa=nxa16
      nri=16
  270 n2=n2+2*nadd
      write(6,275)nri
  275 format(' blocking rows =',i3)
      if(n1.gt.2048.or.n2.gt.2048) go to 320
      nxa=nxa+2*nadd
      id2=n1
      if(n2.gt.n1)id2=n2
      id2=2*id2
c  Blocking parameters for fft are set; call subroutine.
      n22=2*n2
      open(13,access='direct',status='old',
     1 form='unformatted',file=ifile1,
     2 recl=n22*4)
      open(14,access='direct',status='old',
     1 form='unformatted',file=ifile2,
     2 recl=n22*4)
      open(12,access='direct',status='new',
     1 form='unformatted',file=ofile,
     2 recl=n22*4)
      call modify(g1,g2,n2,n1,n22,dy,dx,iopr)
      go to 99
320   write(6,321)
321   format(1x,'Extended rows/columns exceeds 2048; run aborted.')
99    continue
      close(14)
      close(12)
      stop
      end
      subroutine modify(g1,g2,n2,n1,n22,dy,dx,iopr)
c  Note:  In contrast with standard-grid notation, the x,dx,v1
c  direction is north; y,dy,v2 direction is east, and angles are
c  measured clockwise from the +x (north) direction.
      dimension g1(n22),g2(n22)
      ncp1=n2+1
      cn=1./(n2*dy)
      rn=1./(n1*dx)
      scale=float(n1)*float(n2)
      nrnq=float(n1)/2.+1.0000001
      ncnq=float(n2)/2.+1.0000001
      pi=3.1415927e0
      opr=iopr
c  If necessary, SUPPLY parameter or parameter read here.
c
c  Read fftfil.coef file and modify.
c   First row, DC term treated separately.
      jr=1
      read(13,rec=jr)g1
      read(14,rec=jr)g2
c   DC term.
      g1(1)=g1(1)+opr*g2(1)
      g1(2)=g1(2)+opr*g2(2)
c
c   Rest of row 1.
      do 1 i=2,n2
      ic=2*i
      ii=i-1
      if(i.gt.ncnq) ii=-(ncp1-i)
      v2=float(ii)*cn
c    (If necessary, here consider effect of sign of v2.)
      g1(ic-1)=g1(ic-1)+opr*g2(ic-1)
      g1(ic)=g1(ic)+opr*g2(ic)
1     continue
      write(12,rec=jr)g1
c
c  Read rest of fftfil.cof file, row-wise, and modify.
      do 3 jr=2,nrnq
      jj=jr-1
      v1=float(jj)*rn
      v1sq=v1*v1
      read(13,rec=jr)g1
      read(14,rec=jr)g2
      do 2 i=1,n2
      ic=2*i
      ii=i-1
      if(i.gt.ncnq) ii=-(ncp1-i)
      v2=float(ii)*cn
      v2sq=v2*v2
      w=sqrt(v1sq+v2sq)
c  (If necessary, consider effect of sgn of v1, v2.)
      g1(ic-1)=g1(ic-1)+opr*g2(ic-1)
      g1(ic)=g1(ic)+opr*g2(ic)
2     continue
      write(12,rec=jr)g1
3     continue
c
c  Fold into lower two quadrants.
      nrp2=n1+2
      ncp2=n2+2
      do 5 i=2,nrnq-1
      ir=i
      read(12,rec=ir)g1
      ik=nrp2-i
      g1(2)=-g1(2)
      do 4 j=2,ncnq
      jc=2*j
      jk=ncp2-j
      jkc=2*jk
      gr=g1(jc-1)
      gi=-g1(jc)
      g1(jc-1)=g1(jkc-1)
      g1(jc)=-g1(jkc)
      g1(jkc)=gi
      g1(jkc-1)=gr
4     continue
      write(12,rec=ik)g1
5     continue
      return
      end
